/*
   Example Code for the Dr. Duino Starter Kit for Arduino

   This code implements a Larson Scanner with adjustable
   color and speed.

   A Larson scanner is a row of lights which illuminates
   in a back and forth manner. The Larson scanner is named after Glen A. Larson,
   the man responsible for producing both the original Battlestar
   Galactica and Knight Rider television shows where the scanners were
   used on the Cylons robots and on the Knight Rider car.

   Potentiometer    Analog Input              Function
   ==========================================================
        RV6             A0             Red   Color Adjustment
        RV5             A1             Green Color Adjustment
        RV4             A2             Blue  Color Adjustment
        RV1             A5                       Speed

   NOTE: Make sure JP17 and JP18 on the starter kit are in the upper
   positions or completely removed or else you won't be able to download
   code to the starter kit.

   Written by: Craig A. Lindley
   Last Update: 09/29/2019
*/

#include <Adafruit_NeoPixel.h>

// Make sure the jumper for pin 13 is towards the dot on JP12
#define NEOPIXEL_PIN  13
#define NEOPIXEL_COUNT 8

// Assign the color adjustment functions to the analog input potentiometers
#define RED_ADJ A0
#define GRN_ADJ A1
#define BLU_ADJ A2

// Assign the speed function to the analog input potentiometers
#define SPEED_ADJ A5

// Delay values to control speed
#define MIN_DELAY_MS  50
#define MAX_DELAY_MS 500

// Instantiate the NeoPixel driver
Adafruit_NeoPixel pixels(NEOPIXEL_COUNT, NEOPIXEL_PIN, NEO_GRB + NEO_KHZ800);

// Misc variables
unsigned long futureTime;
int pixelNumber;
bool countingUp;

// Setup code goes here
void setup() {

  // Initialize NeoPixel object
  pixels.begin();

  // Initialize variables
  futureTime = 0;
  countingUp = true;
  pixelNumber = 0;
}

// Main code goes here
void loop() {

  // First read the user adjustable values for color and speed

  // Read the color component values
  byte red = analogRead(RED_ADJ);
  byte grn = analogRead(GRN_ADJ);
  byte blu = analogRead(BLU_ADJ);

  // Combine to form a color value
  uint32_t pixelColor = pixels.Color(red, grn, blu);

  // Read the speed value
  byte speedValue = analogRead(SPEED_ADJ);

  // Map analog value 0..254 to MIN_DELAY_MS..MAX_DELAY_MS
  int currentDelayMS = map(speedValue, 0, 254, MIN_DELAY_MS, MAX_DELAY_MS);

  // Is it time for a change ?
  if (futureTime < millis()) {
    // Turn off all pixels
    pixels.clear();

    // Set the current pixel to the current color
    pixels.setPixelColor(pixelNumber, pixelColor);

    if (countingUp) {
      pixelNumber++;
      if (pixelNumber >= 7) {
        countingUp = false;
      }
    } else {
      pixelNumber--;
      if (pixelNumber <= 0) {
        countingUp = true;
      }
    }
    // Make the affected pixel visible
    pixels.show();

    // Calculate future time for next change
    futureTime = millis() + currentDelayMS;
  }
}
